/*
 * MSC (MMC/SD controller) peripheral support.
 *
 * Copyright (C) 2024 Paul Boddie <paul@boddie.org.uk>
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License as
 * published by the Free Software Foundation; either version 2 of
 * the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 51 Franklin Street, Fifth Floor,
 * Boston, MA  02110-1301, USA
 */

#pragma once

#include <l4/devices/msc-generic.h>
#include <l4/sys/types.h>
#include <stdint.h>



#ifdef __cplusplus

#include <l4/devices/dma-x1600.h>

// MMC/SD controller channel.

class Msc_x1600_channel : public Msc_channel
{
protected: 
  Dma_channel *_dma;
  enum Dma_x1600_request_type _request_type_in, _request_type_out;

  // Special overridden method.

  void reset();

public:
  explicit Msc_x1600_channel(l4_addr_t msc_start, l4_addr_t addr,
                             l4_cap_idx_t irq, Cpm_chip *cpm,
                             enum Clock_identifiers clock,
                             Dma_channel *dma,
                             enum Dma_x1600_request_type request_type_in,
                             enum Dma_x1600_request_type request_type_out);

  uint32_t transfer(l4re_dma_space_dma_addr_t from_paddr,
                    l4re_dma_space_dma_addr_t to_paddr,
                    bool recv, uint32_t count);
};

// MMC/SD controller device control.

class Msc_x1600_chip : public Msc_chip
{
protected:
  const enum Dma_x1600_request_type _in_types[2] =
    {Dma_request_msc0_in, Dma_request_msc1_in};

  const enum Dma_x1600_request_type _out_types[2] =
    {Dma_request_msc0_out, Dma_request_msc1_out};

  const enum Clock_identifiers _clocks[2] =
    {Clock_msc0, Clock_msc1};

  unsigned int num_channels()
  { return 2; }

public:
  explicit Msc_x1600_chip(l4_addr_t msc_start, l4_addr_t start, l4_addr_t end,
                          Cpm_chip *cpm);

  Msc_channel *get_channel(uint8_t channel, l4_cap_idx_t irq,
                           Dma_channel *dma);
};

Msc_chip *x1600_msc_chip(l4_addr_t msc_start, l4_addr_t start, l4_addr_t end,
                         Cpm_chip *cpm);

#endif /* __cplusplus */

/* C language interface. */

EXTERN_C_BEGIN

void *x1600_msc_init(l4_addr_t msc_start, l4_addr_t start, l4_addr_t end,
                     void *cpm);

void *x1600_msc_get_channel(void *msc, uint8_t channel, l4_cap_idx_t irq,
                            void *dma);

struct msc_card *x1600_msc_get_cards(void *msc_channel);

uint8_t x1600_msc_num_cards(void *msc_channel);

void x1600_msc_enable(void *msc_channel);

uint32_t x1600_msc_read(void *msc_channel, uint8_t card,
                        struct dma_region *region,
                        uint32_t address, uint32_t count);

uint32_t x1600_msc_read_blocks(void *msc_channel, uint8_t card,
                               struct dma_region *region,
                               uint32_t block_address, uint32_t block_count);

EXTERN_C_END
